unit uToyLibrary;

interface

uses
  classes                   // for TStrings & TStringList
  , uToy                    // for TToy
  , SingletonInheritMany
  ;

type

  TToyLibrary = class(TSingleton)
  private

    FFactoryList: TStringList;

  protected

    procedure Initialise; override;
    procedure Cleanup; override;

  public

    function GetSet1(ToyType: string): TToy;
    function GetSet2(ToyType: string): TToy;
    function GetSet3(ToyType: string): TToy;
    procedure GetToyTypes(ToyTypes: TStrings);
    procedure Register(ToyType, ToyFactoryClass: string);
    procedure Return(Toy: TToy; ToyType: string);

  published

  end;


implementation

uses
  sysutils                        // for Exception
  , uToyFactory
  , uBricksFactory
  , uDollsFactory
  ;


{ TToyLibrary }

procedure TToyLibrary.Cleanup;
var
  nFactory: integer;
begin
  for nFactory := 0 to Pred(FFactoryList.Count) do
    TToyFactory(FFactoryList.Objects[nFactory]).Free;
  FFactoryList.Free;
  inherited;
end;

function TToyLibrary.GetSet1(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet1
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

function TToyLibrary.GetSet2(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet2
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

function TToyLibrary.GetSet3(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet3
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

procedure TToyLibrary.GetToyTypes(ToyTypes: TStrings);
begin
  // Copy the list of toy types to the supplied string list
  ToyTypes.AddStrings(FFactoryList);
end;

procedure TToyLibrary.Initialise;
begin
  FFactoryList := TStringList.Create;
end;

procedure TToyLibrary.Register(ToyType, ToyFactoryClass: string);
type
  TToyFactoryClass = class of TToyFactory;
var
  pcTemp: TToyFactoryClass;           // this is to make sure the factory's Create method is called
  fTemp: TToyFactory;
begin
  pcTemp := TToyFactoryClass(FindClass(ToyFactoryClass));
  fTemp := pcTemp.Create;
  FFactoryList.AddObject(ToyType, fTemp);
end;

procedure TToyLibrary.Return(Toy: TToy; ToyType: string);
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    TToyFactory(FFactoryList.Objects[nFactory]).Return(Toy)
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

end.
